#!/usr/bin/python

program_name = "pfm"
program_version = "0.2"

#
# TODO:
# - GTK can't yet do?
#   - read cursor position!
#   - set cursor to prev directory when going to ".."!
#   - set cursor to prev position when reloading, updating, etc
#   - jump to file by first letter (can't set cursor position..)
#
# - truncate label at bottom so it doesn't make the window resize?
# - make delete/rename windows fit better on small screens
#   (buttons overlap, entry goes past end of window, etc)
# - remove geeteekay widget classes?
# - add key shortcuts to all dialog window options
# - option for "deselect all after launching"
#
# + delete file(s)
#   * with confirmation
#   - just do it
# * Rename file(s)
# - Copy/move file(s)
#   - Use x11 clipboard to send filenames between pfm windows?
#
# - prompt for, and automatically save actions?
#


num_dirs = 0

from gtk import *

def runqueue():
  while events_pending():
    mainiteration()

### BEGIN geeteekay widgets and utils ###

def Run(*args):
  mainloop()

def Quit(*args):
  mainquit()

_ButtonChoice_clicked = None

def _ButtonChoiceButtonCB(button):
  global _ButtonChoice_clicked
  _ButtonChoice_clicked = button
  mainquit()

def ButtonChoiceWindow(title, text, buttons, default=0, long=0,
                       transient=None):
  global _ButtonChoice_clicked
  _ButtonChoice_clicked = None

  win = GtkWindow()
  if transient:
    win.set_transient_for(transient)
  win.connect("destroy", mainquit)
  win.set_position(WIN_POS_MOUSE)
  win.set_modal(1)
  win.set_title(title)
  win.set_border_width(5)

  v = GtkVBox()

  #if len(text) < 300:
  if not long:

    l = GtkLabel(text)
    l.set_line_wrap(1)
    l.show()
    v.add(l)
    
  else:  #if len(text) > 300:
    
    lbox = GtkHBox()
    l = GtkText(None, None)
    l.set_word_wrap(1)
    l.set_line_wrap(1)
    scroll = GtkVScrollbar(l.get_vadjustment())
    lbox.add(l)
    lbox.add(scroll)
    l.insert(None, None, None, text)
    l.set_usize(400, 300)
    v.add(lbox)
    

  box = GtkHButtonBox()
  box.set_border_width(10)


  buttonlist = []
  i = 0
  for item in buttons:
    if type(item) == type((1,2)):
      label, value = item
    else:
      label = value = item


    b = GtkButton(label)
    buttonlist.append([label, value, b])
    b.connect("clicked", _ButtonChoiceButtonCB)

    if default >= 0  and  i == default:
      b.set_flags(CAN_DEFAULT)
      b.set_flags(HAS_DEFAULT)
      box.add(b)
    #if i != default  and  default >= 0:
    #  #box.add(b)
    #  f = GtkFrame()
    #  f.set_border_width(2)
    #  f.set_shadow_type(SHADOW_NONE)
    #  f.add(b)
    #  box.add(f)
    else:
      f = GtkFrame()
      f.set_border_width(2)
      f.set_shadow_type(SHADOW_NONE)
      f.add(b)
      box.add(f)

    i = i + 1

  box.show()
  v.add(box)

  v.show()
  win.add(v)

  win.show_all()
  mainloop()

  retval = None

  for item in buttonlist:
    label, value, b = item
    if b == _ButtonChoice_clicked:
      retval = value

  win.hide()
  win.unrealize()
  del win

  return retval

def EntryWindow(title, text, entry, buttons = ["Ok", "Cancel"], hidden=0, 
                transient=None):
  win = Window(title)
  win.set_modal(1)
  win.w.set_position(WIN_POS_MOUSE)
  if transient:
    win.w.set_transient_for(transient)

  l = Label(text)
  l.set_padding(5)
  win.add(l)

  if type(entry) == type("Hi"):
    e = Entry("", entry, hidden=hidden)
  elif type(entry) == type((1,2)):
    label, text = entry
    e = Entry(text, label, hidden=hidden)
  e.set_padding(5)
  win.add(e)

  bbar = ButtonBar(buttons)
  bbar.set_padding(5)
  win.add(bbar)
  

  e.w.grab_focus()
  win.show()
  Run()
  win.hide()
  win.w.unrealize()

  retval = None

  entered = e.get_value()
  clicked = bbar.get_value()

  retval = (clicked, entered)

  return retval

class Widget:
  """Widget class...
  
  Conventions:
   - classes derived from Widget should name their primary GTK+
     widget "self.w".  This allows callbacks to work.
   - classes derived from Widget should call "self._init()" in
     their __init__()."""
  def __init__(self):
    self._init()

  def _init(self):
    self._cb = {}
    self._cb_func = {
      "pressed" : self._pressed_cb,
      "released" : self._released_cb,
      "clicked" : self._clicked_cb,
      "toggled" : self._toggled_cb,
      "enter" : self._enter_cb,
      "leave" : self._leave_cb,
      "destroy" : self._destroy_cb,
      "" : None
      }


  def _pressed_cb(self, *args):
    self._callback("pressed")

  def _released_cb(self, *args):
    self._callback("released")

  def _clicked_cb(self, *args):
    self._callback("clicked")

  def _toggled_cb(self, *args):
    global _object_table
    (widget, ) = args
    value = widget.get_active()
    self._callback("toggled", value)

  def _enter_cb(self, *args):
    self._callback("enter")

  def _leave_cb(self, *args):
    self._callback("leave")

  def _destroy_cb(self, *args):
    self._callback("destroy")

  def _callback(self, event, *args):
    if self._cb.has_key(event):
      parameters = (self, event) + args
      #print "Calling %s cb...  Args: %s" % (event, repr(parameters))
      apply(self._cb[event], parameters)
      #print "Calling %s cb... done.\n" % event
  
  def set_cb(self, event, cb):
    if self._cb_func.has_key(event)  and  hasattr(self, "w"):
      self._cb[event] = cb
      self.w.connect(event, self._cb_func[event])
    else:
      raise ValueError, "Invalid event type %s" % event

  def run(self):
    Run()

  def show(self):
    if hasattr(self, "w"):
      self.w.show()
    
  def hide(self):
    if hasattr(self, "w"):
      self.w.hide()

  def set_padding(self, xpad, ypad=None):
    if ypad == None:
      ypad = xpad
    if hasattr(self, "w"):
      if hasattr(self.w, "set_padding"):
        self.w.set_padding(xpad, ypad)
      elif hasattr(self.w, "set_border_width"):
        self.w.set_border_width(xpad)

class Label(Widget):
  def __init__(self, text=""):
    self._init()
    self.w = GtkLabel(text)
    self.w.set_line_wrap(1)

  def set_text(self, text):
    self.w.set_text(text)

class Entry(Widget):
  def __init__(self, text=None, label=None, hidden=0):
    self._init()
    self.w = GtkHBox()
    self.e = GtkEntry()
    self.l = None
    if label:
      self.l = GtkLabel(label)
      self.w.add(self.l)
    if text:
      self.set_text(text)
    self.w.add(self.e)
    if hidden:
      self._set_visible(0)

  def set_text(self, text):
    self.e.set_text(text)

  def _set_visible(self, visible):
    self.e.set_visibility(visible)

  def get_value(self):
    return self.e.get_text()

class Button(Widget):
  def __init__(self, text=None):
    self._init()
    self.w = GtkButton()
    if text:
      self.set_text(text)

  def set_text(self, text):
    if hasattr(self, "l"):
      self.l.set_text(text)
    else:
      self.l = GtkLabel()
      self.l.set_line_wrap(1)
      self.l.set_padding(5, 3)
      self.l.set_text(text)
      self.w.add(self.l)

class ButtonBar(Widget):
  def __init__(self, buttons=[]):
    self._init()
    self.w = GtkHButtonBox()
    self.w.set_layout_default(BUTTONBOX_SPREAD)
    self.buttons = []
    self._clicked = None
    for button in buttons:
      if type(button) == type((1,2)):
        text, value = button
      else:
        text = value = button
      b = Button(text)
      b.set_cb("clicked", self._default_cb)
      self.buttons.append((b, value))
      self.w.add(b.w)

  def set_cb(self, which, cb):
    try:
      (button, value) = self.buttons[which]
    except:
      raise IndexError, "Button out of range"
    button.set_cb("clicked", cb)

  def _default_cb(self, object, event):
    for item in self.buttons:
      (button, value) = item
      if button == object:
        self._clicked = value
        #print "Buttonbar clicked: %s" % value
        Quit()

  def get_value(self, result=None):
    return self._clicked

class Window(Widget):
  def __init__(self, title="", modal=0):
    self._init()
    self.w = GtkWindow()
    self.w.set_position(WIN_POS_CENTER)
    self.w.connect("destroy", mainquit)
    self.w.set_title(title)
    self.w.set_modal(modal)
    self.box = GtkTable(99,99)
    self.w.add(self.box)
    self._current = 0
    #self.w.set_border_width(5)

  def set_title(self, title):
    self.w.set_title(title)

  def set_modal(self, modal):
    self.w.set_modal(modal)

  def show(self):
    self.w.show_all()

  def hide(self):
    self.w.hide()

  def add(self, object, x=-1, y=-1, expand=0, gravity=None, padding=1):
    if object  and  hasattr(object, "w"):
      #self.box.add(object.w)
      if x>=0  and  y>=0:
        if expand:
          self.box.attach(object.w, x, x+1, y, y+1)
        else:
          self.box.attach(object.w, x, x+1, y, y+1, xoptions=0, yoptions=0)
      else:
        self.box.attach(object.w, 0, 1, self._current, self._current+1)
        #                xoptions=0, yoptions=0)
        self._current = self._current + 1

  def run(self, *args):
    self.show()
    Run()

  def quit(self, *args):
    Quit()

  def __del__(self):
    self.w.hide()
    self.w.unrealize()
    del self.w

### END geeteekay widgets and utils ###

def num_unit(num):
  bytes = num
  unit = ""
  if bytes > 9999:
    unit = "K"
    bytes = bytes / 1024
  if bytes > 9999:
    unit = "M"
    bytes = bytes / 1024
  if bytes > 9999:
    unit = "G"
    bytes = bytes / 1024
  if bytes > 9999:
    unit = "T"
    bytes = bytes / 1024

  return "%s%s" % (bytes, unit)

class config:
  "Config file handling, and defaults..."
  is_config = 1

  def __init__(self):
    import os
    self.options = {}
    self.usrfilename = "%s/.%s.conf" % (os.environ["HOME"], program_name)
    self.globalfilename = "/etc/%s/%s.conf" % (program_name, program_name)
    self.namespace = {
      "options" : self.options,
      }

  def load(self):
    import os.path
    for name in [self.globalfilename, self.usrfilename]:
      if os.path.exists(name):
        self.parse(name)

  def parse(self, filename):
    #print "parsing %s..." % filename
    try:
      execfile(filename, self.namespace)
    except:
      print "Error in config file %s" % filename
      import traceback, sys
      foo = traceback.format_exception(sys.exc_type,
                                       sys.exc_value,
                                       sys.exc_traceback)
      for item in foo:
        item = item[:-1]
        print item
      print "Press Enter to continue..."
      raw_input()

class file:
  def __init__(self):
    # basename
    # type: dir, file, executable
    # size
    # dates: mtime, ctime, atime
    # more?
    pass
      
class dir:
  def __init__(self, cfg, path="."):
    self.path = path
    self.cfg = cfg
    self.files = []
    self.selected = []
    self.sorttype = "normal"

    self.typeorder = {
      " " : -10,
      "/" : 0,
      "f" : 10,
      }


  def loaddir(self, path=""):
    import os
    import os.path
    import stat
    
    if path:
      if path[0] == "/":
        self.path = path
      else:
        self.path = "%s/%s" % (self.path, path)


      self.path = os.path.abspath(self.path)
      
    self.files = []
    filenames = os.listdir(self.path)
    if filenames == []:
      filenames = [".."]
    #print filenames

    for name in filenames:
      f = file()
      f.basename = name
      fullpath = "%s/%s" % (self.path, name)
      try:
        s = os.stat(fullpath)
        m = s[stat.ST_MODE]
        # TODO: support more file types
        if stat.S_ISDIR(m):
          f.type = "/"
        elif stat.S_ISREG(m):
          f.type = "f"
        else:
          f.type = " "

        f.size = s[stat.ST_SIZE]
      except:
        f.size = 0
        f.type = " "

      self.files.append(f)

    if self.sorttype == "normal":
      self.files.sort(self._sorter)
    elif self.sorttype == "random":
      self.files.sort(self._rand_sorter)

    #for f in self.files:
    #  print "%s: %sb (%s)" % (f.basename, f.size, f.type)

  def _sorter(self, a, b):
    if a.type == b.type:
      if a.basename < b.basename: return -1
      else: return 1
    else:
      return self.typeorder[a.type] - self.typeorder[b.type]

  def _rand_sorter(self, a, b):
    if a.type == b.type:
      import whrandom
      return whrandom.randint(-100, 100)
    else:
      return self.typeorder[a.type] - self.typeorder[b.type]

class dirwin:
  def __init__(self, cfg, path="."):
    self.cfg = cfg
    self.dir = dir(cfg, path)

    self._updating_now = 0
    self._selected = []
    self._cursor = 0

    self.win = GtkWindow()
    self.win.connect("destroy", self.destroy)
    self.win.set_title("pfm") # TODO: add dir to title
    self.win.set_default_size(self.cfg.options["width"],
                              self.cfg.options["height"])

    self.vbox = GtkVBox()
    self.win.add(self.vbox)

    self.make_menus()

    self.sw = GtkScrolledWindow()
    self.sw.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
    self.vbox.pack_start(self.sw, fill=TRUE, expand=TRUE)

    self.status = GtkLabel()
    self.status.set_justify(JUSTIFY_LEFT)
    a = GtkAlignment(0.0, 0.5, 0.0, 0.0)
    a.add(self.status)
    self.vbox.add(a, fill=0, expand=0)

    self._shift_actions = {
      #GDK.Up     : self._key_up,
      #GDK.Down   : self._key_down,
      GDK.Return : self._nop, # handled by menu
      GDK.Right  : self._activate_other,
      }

    self._key_actions = {
      GDK.Left   : self._up_dir,
      GDK.Right  : self._activate,
      GDK.Return : self._nop, # handled by menu
      #GDK.space  : self._file_info,
      GDK.Up     : self._key_up,
      GDK.Down   : self._key_down,
      #GDK.Next   : self._key_next,
      }

    self.clist = GtkCList(1)
    self.clist.set_column_auto_resize(0,1)
    self.sw.add(self.clist)
    self.clist.connect("select-row", self._item_queue_cb)
    self.clist.connect("unselect-row", self._item_dequeue_cb)
    self.clist.connect("key-press-event", self._clist_keypress)
    self.clist.connect("set-focus-child", self._focus_child_cb)
    #self.clist.set_selection_mode(SELECTION_BROWSE)
    self.clist.set_selection_mode(SELECTION_MULTIPLE)

  def make_menus(self):
    self.ag = GtkAccelGroup()
    self.win.add_accel_group(self.ag)
    self.itemf = GtkItemFactory(GtkMenuBar, "<main>", self.ag)

    ltext = "Launch"
    itext = "Choose..."
    if self.cfg.options["activate interactive"]:
      ltext = "Choose..."
      itext = "Launch"

    # Keypresses for quick access to commands...
    ks = [
      ("/File",       None, None, 0, "<Branch>"),
      ("/File/Re_load",  "<Control>L", self._reload, 0, ""),
      ("/File/_New View",  "<Control>N", self._spawn_view, 0, ""),
      ("/File/_Quit",  "<Control>Q", self.destroy, 0, ""),
      ("/Edit",       None, None, 0, "<Branch>"),
      ("/Edit/(de)Select _All",  "<Control>A", self._select_all_cb, 0, ""),
      ("/Edit/_Rename",  "<Control>R", self._rename_cb, 0, ""),
      ("/Edit/_Delete",  "<Control>D", self._delete_cb, 0, ""),
      ("/Edit/(un)Randomi_ze",  "<Control>Z", self._random_order_cb, 0, ""),
      ("/Action",       None, None, 0, "<Branch>"),
      ("/Action/%s" % (ltext), "Return", self._activate, 0, ""),
      ("/Action/%s" % (itext), "<Shift>Return", self._activate_other, 0, ""),
      ]
    self.itemf.create_items(ks)
    self.menubar = self.itemf.get_widget("<main>")
    self.vbox.pack_start(self.menubar, fill=0, expand=0)

  def update_gui(self):
    #print "update_gui()"
    self.clist.clear()

    runqueue()

    self.clist.freeze()
    for f in self.dir.files:
      title = f.basename
      if f.type == "/":
        title = title + "/"
      elif f.type == "*":
        title = title + "/"
      data = [title]
      self.clist.append(data)
    self.clist.thaw()

  def go_dir(self, path="."):
    olddir = self.dir.path
    
    self.dir.loaddir(path)
    #print "go_dir(%s)" % (self.dir.path)
    self._cursor = 0
    self._selected = []

    if path == "..":
      import os.path
      dirs = os.path.split(olddir)
      olddir = dirs[-1]
      #print "Old dir: %s" % (olddir)

      
    self.update()

    if path == "..":
      self.select_by_name(olddir)

  def select_by_name(self, path):
    # TODO: make it work so it can match based on just the first letter
    #       for fast selection of files...
    self._cursor = 0
    return
  
    print "select_by_name(%s)" % (path)
    self._cursor = 0
    for i in xrange(len(self.dir.files)):
      compare = self.dir.files[i].basename
      print "%s <--> %s" % (path, compare)
      if path >= compare:
        self._cursor = i

    print "Setting cursor to... %s" % (self._cursor)
    self.clist.moveto(self._cursor, 0)
    self.clist.select_row(self._cursor, 0)

  def run(self):
    global num_dirs

    self.win.show_all()
    self.go_dir()
    self.clist.grab_focus()
    num_dirs = num_dirs + 1

  def update(self, *args):
    # Don't interrupt myself due to double-clicks
    if self._updating_now:
      return
    self._updating_now = 1  # cheapass mutex

    #self.status.set_text("Hello.")
    self._file_info()

    self.update_gui()
    self.win.set_title("pfm: %s" % (self.dir.path))

    self._updating_now = 0

  def _reload(self, *args):
    self.go_dir()

  def _spawn_view(self, *args):
    d = dirwin(self.cfg, self.dir.path)
    d.run()

  def _select_all_cb(self, *args):
    if len(self._selected) > 0:
      self._unselect_all()
    else:
      self._select_all()

  def _select_all(self, *args):
    self.clist.select_all()

  def _unselect_all(self, *args):
    self.clist.unselect_all()

  def _rename_cb(self, *args):
    self._rename()

  def _rename(self, *args):
    myfiles = self.get_selected_files()

    changed = 0
    for f in myfiles:
      fpath = "%s/%s" % (self.dir.path, f.basename)
      #print "Rename: %s" % (fpath)
      
      oldname = f.basename
      newname = f.basename
      (clicked, entered) = EntryWindow("Rename", oldname, ("", newname), transient=self.win)
      if clicked == "Ok":
	newname = entered
	if newname == "":
	  newname = oldname

      if oldname != newname:
	import os

	os.rename(fpath, "%s/%s" % (self.dir.path, newname))
	f.basename = newname
	changed = 1

    if changed: 
      #self.update()
      self.go_dir(".")

  def _delete_cb(self, *args):
    self._delete()

  def _delete(self, *args):
    myfiles = self.get_selected_files()
    buttons = ["Yes", "No"]
    if len(myfiles) > 1:
      buttons = ["Yes", "No", "Cancel"]

    changed = 0
    for f in myfiles:
      fpath = "%s/%s" % (self.dir.path, f.basename)
      #print "Delete: %s" % (fpath)
      
      clicked = ButtonChoiceWindow("Delete?", "Delete %s?" % (f.basename),
                                   buttons, transient=self.win)
      if clicked == "Cancel":
	break

      elif clicked == "Yes":
	import os

	if f.type == "/":
	  os.rmdir(fpath)
	else:
	  os.unlink(fpath)
	changed = 1

    if changed: 
      #self.update()
      self.go_dir(".")

  def _random_order_cb(self, *args):
    if self.dir.sorttype == "random":
      self.dir.sorttype = "normal"
    else:
      self.dir.sorttype = "random"
    self.go_dir(".")
  
  def _focus_child_cb(self, clist, widget, *args):
    print "Focus: %s" % (widget)

  def _item_select_cb(self, clist, row, col, gdkevent):
    #self.cursor = self.dir[row]
    foo = self.dir.files[row]
    #print "Current: %s" % (foo.basename)

  def _item_queue_cb(self, clist, row, col, gdkevent):
    #self.cursor = self.dir[row]
    #self._cursor = row
    foo = self.dir.files[row]
    self._selected.append(foo)
    #self._file_info()
    #print "Select: %s" % (foo.basename)

  def _item_dequeue_cb(self, clist, row, col, gdkevent):
    #self.cursor = self.dir[row]
    #self._cursor = row
    foo = self.dir.files[row]
    try:
      self._selected.remove(foo)
    except ValueError:
      pass
    #print "Deselect: %s" % (foo.basename)
    #self._file_info()

  def _clist_keypress(self, cl, event, *args):
    if event.state & GDK.SHIFT_MASK:
      #print "Shift"
      actions = self._shift_actions
    else:
      actions = self._key_actions
    
    if actions.has_key(event.keyval):
      actions[event.keyval]()
    else:
      pass
      #print "Keypress: %s" % (event)

  def _nop(self, *args):
    pass

  def _key_up(self, *args):
    if self._cursor > 0:
      self._cursor = self._cursor - 1
    #print "Cursor: %s" % (self._cursor)
    self._file_info()

  def _key_down(self, *args):
    if self._cursor < len(self.dir.files) - 1:
      self._cursor = self._cursor + 1
    #print "Cursor: %s" % (self._cursor)
    self._file_info()

  def _key_next(self, *args):
    self._cursor = self._cursor + 1
    if self._cursor >= len(self.dir.files):
      self._cursor = len(self.dir.files) - 1
    #print "Cursor: %s" % (self._cursor)
    self._file_info()

  def _file_info(self):
    #print "file_info: %s" % (self._cursor)
    f = self.dir.files[self._cursor]
    if f.type == "/":
      text = "%s/ (DIR)" % (f.basename)
    elif f.type == "f":
      hsize = num_unit(f.size)
      text = "%s (%s)" % (f.basename, hsize)
    else:
      text = "%s/ (%s)" % (f.basename, f.type)
    self.status.set_text(text)

  def _up_dir(self, *args):
    #print "Up dir!"
    self.go_dir("..")

  def get_selected_files(self):
    #print "get_selected_files()"
    val = []
    current = self.dir.files[self._cursor]
    if len(self._selected) < 1:
      # return item under the cursor
      val = [current]
    else:
      # return all selected items, plus item under the cursor
      # oops, that's annoying.  just return selected items.
      #for f in self._selected:
      #  print f.basename
      #if current in self._selected:
      #  val.extend(self._selected)
      #else:
      #  val.append(current)
      #  val.extend(self._selected)
      val.extend(self._selected)

    return val

  def _activate_from_menu(self, *args):
    self._activate_auto()

  def _activate(self, *args):
    if self.cfg.options["activate interactive"]:
      self._activate_interactive()
    else:
      self._activate_auto()

  def _activate_other(self, *args):
    if self.cfg.options["activate interactive"]:
      self._activate_auto()
    else:
      self._activate_interactive()

  def _shell_escape(self, path):
    import string

    safe = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ" +\
           "1234567890-_,./"

    skip = 0
    i = 0
    flen = len(path)
    while i < flen:
      if path[i] not in safe:
	path = path[:i] + "\\" + path[i:]
	flen = flen + 1
	i = i + 1
	skip=1
      i = i + 1

    return path
  
  def _activate_auto(self, program=None):
    #print "Activate!"
    filestring = ""
    myfiles = self.get_selected_files()

    if myfiles[0].type == "/":
      self.go_dir(myfiles[0].basename)
      return

    if not program:
      program = self.get_action(myfiles[0])
    
    for f in myfiles:
      fpath = "%s/%s" % (self.dir.path, f.basename)
      fpath = self._shell_escape(fpath)
      filestring = "%s%s " % (filestring, fpath)
    cmd = "%s %s" % (program, filestring)
    #print cmd

    import os
    if os.fork() != 0:
      os.system(cmd)
      os._exit(1)

  def _activate_interactive(self, *args):
    #print "Activate (interactive)"
    filestring = ""
    myfiles = self.get_selected_files()

    if myfiles[0].type == "/":
      #fpath = self.dir.path + "/" + myfiles[0].basename
      #self._spawn_view(fpath)
      self.go_dir(myfiles[0].basename)
      return

    w = GtkWindow()
    w.set_modal(1)
    w.set_title("Choose Action")
    #w.set_default_size(200,200)
    w.connect("destroy", mainquit)

    vbox = GtkVBox()
    w.add(vbox)

    l = GtkLabel("Choose Action:")
    vbox.add(l, fill=0, expand=0)
    
    #sw = GtkScrolledWindow()
    #vbox.add(sw, fill=1, expand=1)

    cl = GtkCList(1)
    cl.set_column_auto_resize(0,1)
    #sw.add(cl)
    vbox.add(cl)
    cl.connect("select-row", self._activate_interactive_select)
    cl.connect("key-press-event", self._activate_interactive_keypress)
    cl.connect("button-release-event", self._activate_interactive_go)
    cl.set_selection_mode(SELECTION_BROWSE)

    self._activate_interactive_chosen = 0
    self._activate_interactive_cursor = 0
    actions = self.get_actions(myfiles[0])
    for a in actions:
      cl.append([a])
    cl.grab_focus()

    w.set_position(WIN_POS_MOUSE)
    w.show_all()
    mainloop()

    w.hide()
    w.unrealize()
    del w

    if self._activate_interactive_chosen:
      action=actions[self._activate_interactive_cursor]
      self._activate_auto(program=action)

  def _activate_interactive_keypress(self, cl, event, *args):
    if event.keyval in [GDK.Return, GDK.space, GDK.Right]:
      self._activate_interactive_chosen = 1
      mainquit()
    elif event.keyval in [GDK.Escape, GDK.Left]:
      self._activate_interactive_chosen = 0
      mainquit()
    
  def _activate_interactive_go(self, cl, event, *args):
    self._activate_interactive_chosen = 1
    mainquit()
      
  def _activate_interactive_select(self, cl, row, *args):
    self._activate_interactive_cursor = row
    
  def get_actions(self, f):
    import re

    actions = self.cfg.options["actions"]
    
    i = 0

    acts = []
    
    for i in xrange(len(actions)):
      pattern = actions[i][0]
      command = actions[i][1]
      fpath = "%s/%s" % (self.dir.path, f.basename)
      if re.search(pattern, fpath, re.I):
        acts.append(command)
        #print "Action: %s" % (command)
      
    return acts

  def get_action(self, f):
    acts = self.get_actions(f)
    return acts[0]

  def destroy(self, *args):
    global num_dirs

    self.win.hide()
    self.win.unrealize()
    mainquit()
    del self.win

    num_dirs = num_dirs - 1

def run(args):
  import sys
  args = sys.argv[1:]
  
  cfg = config()
  cfg.load()
  
  if len(args) > 0:
    for a in args:
      p = dirwin(cfg, a)
      p.run()
  else:
    p = dirwin(cfg)
    p.run()

  while num_dirs > 0:
    mainloop()


if __name__ == "__main__":
  import sys
  run(sys.argv[1:])
